// Copyright (C) 2006-2010 Xilinx, Inc.  All rights reserved
// $Id: RevArc.h,v 1.5 2010/11/08 19:03:09 jcorbett Exp $
#ifndef Iv_RevArc_h
#define Iv_RevArc_h

#include "Databases/Rf/export/Facade/Rf_Arc.h"
#include "Databases/Rf/export/Facade/Rf_Node.h"

#include "main.h"

// R#report_interregion_signals - Reversable Arc. We're not
// restricting the search for short paths to the normal current
// direction, but our cost function is not symmetric, i.e. traversing
// an arc forwards may have a different cost from traversing it
// backwards, so this class allows us to keep track of the direction
// an arc is used in a route.
class RevArc {
public:
   typedef enum Direction { Forward, Reverse };
   RevArc(Rf_Arc a, Rf_Node source)
      : arc(a),
        direction(source == a.GetNodes().first
                  ? Forward
                  : Reverse)
   {
      ASSERT((direction == Reverse) == (a.GetNodes().second == source));
   }
   RevArc(Rf_Arc a, RevArc predecessor)
      : arc(a),
        direction(predecessor.GetTarget() == a.GetNodes().first
                  ? Forward
                  : Reverse)
   {
      ASSERT((direction == Reverse)
             == (a.GetNodes().second == predecessor.GetTarget()));
   }
   Rf_Node GetSource() {
      if (direction == Forward) {
         return arc.GetNodes().first;
      } else {
         return arc.GetNodes().second;
      }
   }
   Rf_Node GetTarget() {
      if (direction == Forward) {
         return arc.GetNodes().second;
      } else {
         return arc.GetNodes().first;
      }
   }
   Rf_Node GetOtherNode(Rf_Node node) {
      return arc.GetOtherNode(node);
   }
   std::string GetSourceName() {
      Rf_Node rfNode(GetSource());
      Port_String tile, wire;
      Port::UINT32 wireIndex(rfNode.GetWireIndex(arc));
      rfNode.GetWireNameAndTileInfo(wireIndex, tile, wire);
      return std::string(wire);
   }
   std::string GetTargetName() {
      Rf_Node rfNode(GetTarget());
      Port_String tile, wire;
      Port::UINT32 wireIndex(rfNode.GetWireIndex(arc));
      rfNode.GetWireNameAndTileInfo(wireIndex, tile, wire);
      return std::string(wire);
   }
   Port::UINT32 GetSourceIndex() {
      return GetSource().GetWireIndex(arc);
   }
   Port::UINT32 GetTargetIndex() {
      return GetTarget().GetWireIndex(arc);
   }
   Direction GetDirection() { return direction; }
   // This is an abstraction leak. Use with caution.
   Rf_Arc GetRfArc() { return arc; }
private:
   RevArc();
   Rf_Arc arc;
   Direction direction;
};

#endif
